#include <Arduino.h>
#include <SPI.h>
#include <Wire.h>
#include <avr/pgmspace.h>

#include "dog.h"
#include "font_6x8.h"
#include "font_8x16.h"
#include "Bopla_BLV.h"
#include "EA_BLV.h"

#include "MCP7940.h"
#include "PCT2075.h"

#define LOGOTIME  25
#define DOWNTIME1 1000
#define DOWNTIME2 100

#define XAXIS  50
#define YAXIS  3
#define YSTEPS 10
#define XSTEPS 10

#define MENUX     40
#define MENUY     17
#define MENSTEPS  9

#define T_RETURN A2
#define T_CANCEL A1
#define T_LEFT   A3
#define T_RIGHT   5
#define T_UP      7
#define T_DOWN    6

#define B_RETURN  1
#define B_CANCEL  2
#define B_LEFT    4
#define B_RIGHT   8
#define B_UP      16
#define B_DOWN    32


#define MAIN    0
#define MENU    1
#define M_RATE  2
#define M_THOUR 3
#define M_TMIN  4
#define M_TSEC  5
#define M_LED   6
#define M_CONTR 7



dog DOG;
MCP7940 RTC;
PCT2075 PCT;

const char s_Menu[]   PROGMEM = "Menu";
const char s_Start[]  PROGMEM = "Start";
const char s_Stop[]   PROGMEM = "Stop";
const char s_Main[]   PROGMEM = "Main Menu";
const char s_Rate[]   PROGMEM = "Rate:";
const char s_Time[]   PROGMEM = "Time:";
const char s_Led[]    PROGMEM = "LED:";
const char s_Contr[]  PROGMEM = "Contr:";
const char s_Back[]   PROGMEM = "Back";
const char s_Edit[]   PROGMEM = "Edit";
const char s_Ok[]     PROGMEM = "Ok";



byte led, contr;
byte menu, rising, logging;
char str[20];
unsigned int time_cnt, cnt, timer, downtime, log_index;

void main_screen (void);
void menu_screen (void);

#define str_progmem(s) strcpy_P(str, s)

/*----------------------------
Func: setup()
Desc: Initializes SPI Hardware/Software, Variables and shows start screen
Vars: ---
------------------------------*/
void setup()
{
  byte i;
  RTCTIME newtime;
  //Init Display, RTC and Temperature
  DOG.initialize(10, 0, 0, 9, 4, DOGS102, 3); //SS = 10, 0,0= use Hardware SPI, 9 = A0, 4 = RESET, EA DOGS102-6 (=102x64 dots)
  led = 100;
  DOG.backlight(led);
  contr = 12;
  DOG.contrast(contr);
  RTC.initialize(str);
  PCT.initialize(str);

  //Set variables to default
  logging = false;
  newtime.hour = 0;
  newtime.min  = 0;
  newtime.sec  = 0;
  newtime.day  = 1;
  newtime.month= 1;
  newtime.year = 16;
  newtime.dayofweek = 1;
  //RTC.setTime(&newtime); //normally not necessary as RTC is bufferd with battery. User can change time in menu

  //Set Pin modes
  pinMode(T_RETURN, INPUT_PULLUP);
  pinMode(T_CANCEL, INPUT_PULLUP);
  pinMode(T_LEFT, INPUT_PULLUP);
  pinMode(T_RIGHT, INPUT_PULLUP);
  pinMode(T_UP, INPUT_PULLUP);
  pinMode(T_DOWN, INPUT_PULLUP);

  //Show animated start screen
  i=YPIXEL-1;
  do
  {
    DOG.clear();
    DOG.picture(0,i, Image_Bopla);
    DOG.update();
    delay(LOGOTIME);
  }while(i--);

  i=XPIXEL-1;
  do
  {
    DOG.picture(0,0, Image_Bopla);
    DOG.clear(i,0, XPIXEL-1,YPIXEL-1);
    DOG.picture(i,0, Image_EA);
    DOG.update();
    delay(LOGOTIME);
  }while(i--);
  
  menu = MAIN;
}

/*----------------------------
Func: main_screen()
Desc: draws main window (temperature)
Vars: ---
------------------------------*/
void main_screen(void)
{
  unsigned int ei;
  byte i, dat1, dat2, steps, incr, ie; 
  byte *p_data, *p_logindex;
  
  //Draw coordinate system
  DOG.clear();
  DOG.line(YAXIS, 0, YAXIS, XAXIS + YAXIS);
  DOG.line(0, YAXIS, YAXIS, 0);
  DOG.line(YAXIS, 0, 2 * YAXIS, YAXIS);
  DOG.line(0, XAXIS, XPIXEL - 1, XAXIS);
  DOG.line(XPIXEL - 1 - YAXIS, XAXIS - YAXIS, XPIXEL - 1, XAXIS);
  DOG.line(XPIXEL - 1 - YAXIS, XAXIS + YAXIS, XPIXEL - 1, XAXIS);
  i = XAXIS; //draw unit Y-Axis
  do
  {
    i -= YSTEPS;
    DOG.line(0, i, 2 * YAXIS, i);
  } while (i > YAXIS + YSTEPS);
  i = YAXIS; //draw unit X-Axis
  do
  {
    i += XSTEPS;
    DOG.line(i, XAXIS - YAXIS, i, XAXIS + YAXIS);
  } while (i < XPIXEL - 1 - YAXIS - XSTEPS);

  //Label for buttons
  DOG.string(0, YPIXEL - 1 - 8, font_6x8, str_progmem(s_Menu));
  if(logging == false)
    DOG.string(XPIXEL - 1 - 5 * 6, YPIXEL - 1 - 8, font_6x8, str_progmem(s_Start));
  else
    DOG.string(XPIXEL - 1 - 5 * 6, YPIXEL - 1 - 8, font_6x8, str_progmem(s_Stop));
    
  p_data = PCT.getLog(&ei); //p_data is set to first logged data, ei is set to endindex in function getLog
  if(ei)
  {
    PCT.printTempTime(ei-1); //show last value
    i = strlen(str);
    DOG.string(XPIXEL-i*6, 0, font_6x8, str);
    if(log_index > ei - 1)
      log_index = ei - 1;
    PCT.printTempTime(log_index); //show user value
    i = strlen(str);
    DOG.string(XPIXEL-i*6, 8, font_6x8, str);

    p_logindex = &p_data[log_index];
       
    if(ei == 1) //show graph
    {
      steps = 1;
      incr = 1;
      DOG.line(YAXIS + 1, XAXIS - ((unsigned int)(*p_data * XAXIS) / 180),  YAXIS + 1, XAXIS - ((unsigned int)(*p_data* XAXIS) / 180));
    }
    else
    {
      steps = (XPIXEL-YAXIS) / ei; //calculate x steps to show all available data
      if(steps == 0)
      {
        steps = 1;
        incr = ei / (XPIXEL-YAXIS) + 1;
      }
      else
        incr = 1;
    
      i=0;
      ie = ei/incr-1;
       
      while(i < ie) //show graph
      {   
        dat1 = *p_data;
        p_data += incr;
        dat2 = *p_data;
        DOG.line(YAXIS + i*steps, XAXIS - ((unsigned int)(dat1 * XAXIS) / 180),  YAXIS + ((i+1)*steps), XAXIS - ((unsigned int)(dat2 * XAXIS) / 180));
        i++;
      }
    }

    dat1 = YAXIS + (log_index*steps)/incr;
    dat2 = XAXIS - ((unsigned int)(*p_logindex * XAXIS) / 180);
      
    DOG.line(dat1-3, dat2,   dat1+3, dat2  );
    DOG.line(dat1,   dat2-3, dat1,   dat2+3);
  }    
}

/*----------------------------
Func: menu_screen()
Desc: shows menu screen and reacts to button events in menu
Vars: ---
------------------------------*/
void menu_screen(void)
{
  byte i;
  DOG.clear();
  DOG.string((XPIXEL - 9 * 8) / 2, i, font_8x16, str_progmem(s_Main)); //Header
  //menu points
  i = MENUY;
  DOG.string(0, i, font_6x8, str_progmem(s_Rate));   DOG.string(MENUX, i, font_6x8, PCT.setupLog(0));      i += MENSTEPS;
  DOG.string(0, i, font_6x8, str_progmem(s_Time));   DOG.string(MENUX, i, font_6x8, RTC.timestring());     i += MENSTEPS;
  DOG.string(0, i, font_6x8, str_progmem(s_Led));    DOG.string(MENUX, i, font_6x8, itoa(led, str, 10));   i += MENSTEPS;
  DOG.string(0, i, font_6x8, str_progmem(s_Contr));  DOG.string(MENUX, i, font_6x8, itoa(contr, str, 10)); i += MENSTEPS;

  switch(menu) //edit values
  {
    case M_RATE:
      i = strlen(PCT.setupLog(0));
      DOG.invert(MENUX-1,MENUY-1, MENUX+i*6-1,MENUY-2+MENSTEPS); 
      break;
    case M_THOUR:
      DOG.invert(MENUX-1,MENUY+1*(MENSTEPS-1), MENUX+2*6-1,MENUY-2+2*MENSTEPS);
      break;
    case M_TMIN:
      DOG.invert(MENUX+3*6-1,MENUY+1*(MENSTEPS-1), MENUX+5*6-1,MENUY-2+2*MENSTEPS);
      break;
    case M_TSEC:
      DOG.invert(MENUX+6*6-1,MENUY+1*(MENSTEPS-1), MENUX+8*6-1,MENUY-2+2*MENSTEPS);
      break;
    case M_LED:
      i = strlen(itoa(led, str, 10));
      DOG.invert(MENUX-1,MENUY-1+2*MENSTEPS, MENUX+i*6-1,MENUY-2+3*MENSTEPS);
      break;
    case M_CONTR:
      i = strlen(itoa(contr, str, 10));
      DOG.invert(MENUX-1,MENUY-1+3*MENSTEPS, MENUX+i*6-1,MENUY-2+4*MENSTEPS);
      break;
  }

  //show editmode or standard back button
  if(menu == MENU)
  {
    DOG.string(0, YPIXEL - 1 - 8, font_6x8, str_progmem(s_Back));
    DOG.string(XPIXEL - 1 - 4 * 6, YPIXEL - 1 - 8, font_6x8, str_progmem(s_Edit));
  }
  else
  {
      DOG.string(XPIXEL - 1 - 4 * 6, YPIXEL - 1 - 8, font_6x8, str_progmem(s_Ok));
  }
}



/*----------------------------
Func: loop()
Desc: main loop get button events
Vars: ---
------------------------------*/
void loop()
{
  byte tast;

  tast = (!digitalRead(T_RETURN)  << 0);
  tast |= (!digitalRead(T_CANCEL) << 1);
  tast |= (!digitalRead(T_LEFT)   << 2);
  tast |= (!digitalRead(T_RIGHT)  << 3);
  tast |= (!digitalRead(T_UP)     << 4);
  tast |= (!digitalRead(T_DOWN)   << 5);

  if(tast == 0) //no button pressed
  {
    rising = true;
    time_cnt = 0;  
    timer = (unsigned int)millis();
    downtime = DOWNTIME1; //debouncing
  }
    
  if (tast == 0 && menu == MAIN && rising)    //Main screen
    menu = MAIN;
  else if (tast == B_RETURN && menu == MAIN && rising)    //start / stop counting
  {
    if(logging == false)
    {
      RTC.getTime();
      PCT.startLog(RTC.time.hour, RTC.time.min, RTC.time.sec);
      logging = true;
      log_index = 0;
      attachInterrupt(digitalPinToInterrupt(2), ISR_sec, FALLING); //get RTC interrupts (every 1 second)
    }
    else
    {
      detachInterrupt(digitalPinToInterrupt(2));
      logging = false;
    }
    menu = MAIN;
  }
  else if ((tast == B_LEFT || tast == B_RIGHT) && menu == MAIN && rising)    //show different logged times
  {
    if(tast == B_RIGHT)
       log_index++;
    else if(log_index)
       log_index--;
  }
  else if (tast == B_CANCEL && menu == MAIN && rising)  //From Main screen to menu
    menu = MENU;
  else if (tast == B_CANCEL && menu == MENU && rising)  //Menu  to Main
    menu = MAIN;
  else if (tast == B_RETURN && menu == MENU && rising) //Menu to Edit: Rate
    menu = M_RATE;
  else if (tast == B_RIGHT && menu >= M_RATE && menu <= M_CONTR && rising) //Menu edit change forward
  {
    if(++menu > M_CONTR)
      menu = M_RATE;
  }
  else if (tast == B_LEFT && menu >= M_RATE && menu <= M_CONTR && rising) //Menu edit change backward
  {
    if(--menu < M_RATE)
      menu = M_CONTR;
  }
  else if (tast == B_RETURN && menu >= M_RATE && menu <= M_CONTR && rising) //leave edit mode
    menu = MENU;
  else if (tast == B_UP && menu == M_RATE && rising)  //increase rate
    PCT.setupLog(1);
  else if (tast == B_DOWN && menu == M_RATE && rising)  //decrease rate
    PCT.setupLog(2);
  else if (tast == B_UP  && menu >= M_THOUR && menu <= M_TSEC && rising)  //set time: increase
  {
    RTCTIME temp = RTC.time;

    switch(menu)
    {
      case M_THOUR: temp.hour++;  break;
      case M_TMIN:  temp.min++;   break;
      case M_TSEC:  temp.sec++;   break;
    }
    
    if(temp.hour>23)
      temp.hour=0;
    if(temp.min > 59)
      temp.min = 0;
    if(temp.sec > 59)
      temp.sec = 0;
      
    RTC.setTime(&temp);
  }
  else if (tast == B_DOWN && menu >= M_THOUR && menu <= M_TSEC && rising)  //set time: decrease
  {
    RTCTIME temp = RTC.time;

    switch(menu)
    {
      case M_THOUR: temp.hour--;  break;
      case M_TMIN:  temp.min--;   break;
      case M_TSEC:  temp.sec--;   break;
    }
    
    if(temp.hour>23)
      temp.hour=23;
    if(temp.min > 59)
      temp.min = 59;
    if(temp.sec > 59)
      temp.sec = 59;
      
    RTC.setTime(&temp);
  }
  else if (tast == B_UP && menu == M_LED && rising)  //increase backlight
  {
    if(led < 100)
      led++;
    DOG.backlight(led);
  }
  else if (tast == B_DOWN && menu == M_LED && rising)//decrease backlight
  {
    if(led)
      led--;
    DOG.backlight(led);
  }
    else if (tast == B_UP && menu == M_CONTR && rising)  //increase contrast DOG-Display
  {
    if(contr<63)
      contr++;
    DOG.contrast(contr);
  }
  else if (tast == B_DOWN && menu == M_CONTR && rising) //decrease contrast DOG-Display
  {
    if(contr)
      contr--;
    DOG.contrast(contr);
  }
  
  if(tast)
  {
    rising = false;
    if(((tast == B_DOWN || tast == B_UP) && menu >= M_RATE && menu <= M_CONTR) || ((tast == B_LEFT || tast == B_RIGHT) && menu == MAIN)) //fast change, button down event
    {
      cnt = (unsigned int)millis();
        if (timer != cnt)
        {
          time_cnt += (timer < cnt) ? (cnt - timer) : (0xFFFF - timer + cnt + 1);
          timer = cnt;
        }
        if(time_cnt >= downtime)
        {
          rising = true;
          time_cnt = 0;
          downtime = DOWNTIME2;
        }
    }
    
  }

  switch(menu)
  {
    case MAIN:
      main_screen(); break;
    case MENU:
      menu_screen(); break;
    case M_RATE:
      menu_screen(); break;
    case M_THOUR:
      menu_screen(); break;
    case M_TMIN:
      menu_screen(); break;
    case M_TSEC:
      menu_screen(); break;
    case M_LED:
      menu_screen(); break;
    case M_CONTR:
      menu_screen(); break;
  }
  DOG.update(); //show new screen content
  
}

/*----------------------------
Func: ISR_sec()
Desc: ISR for temp logging. ISR is called every other 1 sec
Vars: ---
------------------------------*/
void ISR_sec(void)
{
  sei();
  RTC.getTime();
  PCT.storeTemp(RTC.time.hour, RTC.time.min, RTC.time.sec);
}

